/*****************************************
* This JMenuBar describes the MenuBar
* of MainFrame
*/


import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.net.URL;

public class FrameMenu extends JMenuBar implements ActionListener{
	private MainFrame parent;
	private InfoBrowser browser;
	private JMenu program, view, customize, options, 
		help, chooseWavelet, chooseFilter, speed;
	private JMenuItem  wave, transformation, quit, index, about;
	private ButtonGroup wavelets, filters, animSpeed;
	private JRadioButtonMenuItem whaar, mexican, morlet,
						fhaar, daub2, slow, medium, fast;
	private JCheckBoxMenuItem blinds;
	private	final JApplet applet;
	
	
	FrameMenu( MainFrame parent, final JApplet applet){
		this.parent = parent;
		this.applet = applet;
		browser = new InfoBrowser();
		
		program = new JMenu("Program");
		view = new JMenu( "View" );
		options = new JMenu("Options");
		customize = new JMenu( "Customize" );
		help = new JMenu("Help");
		chooseWavelet = new JMenu("Choose Wavelet");
		chooseFilter = new JMenu("Choose Filter" );
		speed = new JMenu( "Speed");
		speed.setEnabled( false );
		
		wave = new JMenuItem("Wavelets");
		wave.setEnabled( false );
		transformation = new JMenuItem( "Transformation" );
		quit = new JMenuItem("Quit");
		index = new JMenuItem("Index");
		about = new JMenuItem("About");
		
		wavelets = new ButtonGroup();
		filters = new ButtonGroup();
		animSpeed = new ButtonGroup();
		
		whaar = new JRadioButtonMenuItem("Haar Wavelet", true);
		mexican = new JRadioButtonMenuItem("Mexican Hat Wavelet");
		morlet = new JRadioButtonMenuItem("Morlet Wavelet");
		fhaar = new JRadioButtonMenuItem("Haar Filter", true);
		daub2 = new JRadioButtonMenuItem("Daubechies-2 Filter");
		slow = new JRadioButtonMenuItem("slow");
		medium = new JRadioButtonMenuItem("medium",true);
  	fast = new JRadioButtonMenuItem("fast");
		
		blinds = new JCheckBoxMenuItem( "Red-green Blinds", false );
		blinds.setEnabled( false );
		
		//program menu
		add( program );
		program.add( quit );
		quit.addActionListener( this );
		
		//view menu
		add( view );
		view.add( wave );
		view.add( transformation );
		wave.addActionListener( (ActionListener) parent );
		transformation.addActionListener( (ActionListener) parent );
		
		//customize view
		add( customize );
		customize.add( speed );
		animSpeed.add( slow );
		animSpeed.add( medium );
		animSpeed.add( fast );
		speed.add( slow );
		speed.add( medium );
		speed.add( fast );
		slow.addActionListener( (ActionListener) parent );
		medium.addActionListener( (ActionListener) parent );
		fast.addActionListener( (ActionListener) parent );
		customize.add( blinds );
		blinds.addActionListener( this );
		
		//options menu
		add( options );
		options.add( chooseWavelet );
		chooseWavelet.add( whaar );
		chooseWavelet.add( mexican);
		chooseWavelet.add( morlet );
		wavelets.add( whaar );
		wavelets.add( mexican );
		wavelets.add( morlet );
		whaar.addActionListener( (ActionListener) parent );
		mexican.addActionListener( (ActionListener) parent );
		morlet.addActionListener( (ActionListener) parent );
		options.add( chooseFilter);
		chooseFilter.add( fhaar );
		chooseFilter.add( daub2 );
		chooseFilter.setEnabled(false);
		filters.add( fhaar );
		filters.add( daub2 );
		fhaar.addActionListener( (ActionListener) parent );
		daub2.addActionListener( (ActionListener) parent );
			
		//help menu
		add( help );
		help.add( index );
		help.addSeparator();
		help.add( about );
		index.addActionListener( this );
		about.addActionListener( this );
	}//end of FrameMenu()
	
	
	/******************* actionPerformed*************************
	* ActionEvent_Handling
	*/
	public void actionPerformed( ActionEvent e ){
		if( e.getSource() == blinds ){
			if( blinds.isSelected() ){
				//purple and blue colors
				parent.setColors( new Color( 255, 183, 255 ),
					new Color(219, 219, 255), new Color( 176, 0, 176 ),
						new Color( 0, 0, 219 ), new Color( 253, 0, 253 ));
			}else{
				//red and green colors
				parent.setColors( new Color( 255, 189, 189 ),
					new Color( 202, 255, 202 ), new Color(181,36,57),
						new Color( 0, 128, 0 ), Color.red );
			}
		}else	if ( e.getSource() == quit ){
			parent.dispose();		
		}
		else if ( e.getSource() == about ){
			
			URL u = FrameMenu.class.getResource("Image.jpg");

			Icon image = new ImageIcon( applet.getImage(u));

			
			String message = "Multiresolution Analysis Applet Version 1.0\n\n"+

					  "Author : Julia Schneider\n"+

					  "Student of Computer Science and Business Administration\n"+

					  "\n"+

					  "(c) 2001 University of Mannheim\n "+

					  "Chair of Practical Computer Science IV (PI IV)\n"+

					  "http://www.informatik.uni-mannheim.de/informatik/pi4\n"+

					  "http://www.uni-mannheim.de";
					  	
			JOptionPane.showMessageDialog( 
				null, message, "About", 
				JOptionPane.INFORMATION_MESSAGE, image);
				
		}else if ( e.getSource() == index ){
			
			URL 	appletURL = applet.getCodeBase(); 
			browser.setPage(appletURL, "help/index.html");
			
		}

			
			
	}//end of actionPerformed()
	
	
	
	/***********************setChooseWavelet()*******************
	* choseWavelet and choseFilter must not be enabled
	* at the same time, but only if its Tab is activated
	*/
	public void setChoose( boolean b){
		chooseWavelet.setEnabled( b );
		transformation.setEnabled( b );
		chooseFilter.setEnabled( !b );
		speed.setEnabled( !b );
		blinds.setEnabled( !b );
		wave.setEnabled( !b );
	}

	
}//end of class


			
	