
import wavelets.*;
import transformation.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;


public class MainFrame extends JFrame implements ChangeListener, 
			ActionListener{
	private JApplet parent;
	private FrameMenu menu;
	private final JTabbedPane tPan;
	private JPanel waveletPanel, transformPanel, waveletCenter,
		transformCenter;
	private HeadlinePanel head1, head2;
	private AdjustmentPanel adjust;
	private AnalysePanel analyse;
	private SynthesisPanel synthesis;
	private AnalyseHaarPanel analyseHaar;
	private AnalyseDaub2Panel analyseDaub2;
	private SynthesisHaarPanel synthesisHaar;
	private SynthesisDaub2Panel synthesisDaub2;
	private PlotPanel plot;
	private ButtonPanel button;

	
	public MainFrame( JApplet parent){
		super("Multiresolution Analysis");
		this.parent = parent;
		
		menu = new FrameMenu( this, parent );
		tPan = new JTabbedPane( JTabbedPane.TOP );
		waveletPanel = new JPanel( new BorderLayout() );
		transformPanel = new JPanel( new BorderLayout() );
		waveletCenter = new JPanel( new BorderLayout() );
		transformCenter = new JPanel( new BorderLayout() );
		head1 = new HeadlinePanel( "Haar Wavelet" );
		head2 = new HeadlinePanel( "Haar Filter" );
		plot = new PlotPanel( new HaarWave() );
		adjust = new AdjustmentPanel( plot );
		
		analyseHaar = new AnalyseHaarPanel();
		analyseDaub2 = new AnalyseDaub2Panel();
		synthesisHaar = new SynthesisHaarPanel();
		synthesisDaub2 = new SynthesisDaub2Panel();
		analyse = analyseHaar;
		synthesis = synthesisHaar;
		
		button = new ButtonPanel( analyse, synthesis, this );
		/* shaping the frame
		*/
		setJMenuBar( menu );
		
		Container c = getContentPane();
			
		tPan.add( "Wavelets", waveletPanel );
		waveletPanel.add( adjust, BorderLayout.WEST );
		waveletCenter.add( head1, BorderLayout.NORTH );
		waveletCenter.add( plot, BorderLayout.CENTER );
		waveletPanel.add( waveletCenter, BorderLayout.CENTER );
		
		
		tPan.add( "Transformation", transformPanel );
		transformPanel.add( button, BorderLayout.WEST );
		transformCenter.add( head2, BorderLayout.NORTH );
		transformCenter.add( analyse, BorderLayout.CENTER );
		transformPanel.add( transformCenter, BorderLayout.CENTER );
		
		
		tPan.addChangeListener( this );
		c.add( tPan );
		
		
		/* adapting the framesize to the user's screensize
		*/
		setResizable(true);
		Dimension d = (Toolkit.getDefaultToolkit()).getScreenSize();
		int screenheight = d.height;
 		int screenwidth = d.width;
 		if (screenwidth < 800){
 			//	if the user's actuall screenresolution is smaller than 
 			// 	the standard screensize (800x600), the frame will adapt
 			//	to the actual screensize
			setSize(screenwidth,screenheight); 
		}
		else{ 			
			// the frame is displayed centered
			setSize(800,600);
			setLocation((screenwidth-800)/2,(screenheight-600)/2);
		}

		
		/* activating the CloseButton of the Frame
		*/
		addWindowListener( new WindowAdapter(){
			public void WindowClosing( WindowEvent e ) {
				e.getWindow().dispose();
			}
		});
	
		parent.getAppletContext().showStatus("Frame loaded!");
		
		
	}// end of MainFrame
	
	
	/**
	*this method sets the animationcolors of the 
	*TransformationPanels analyse and synthesis
	*/
	public void setColors( Color backRed, Color backGreen,
		Color gridRed,  Color gridGreen, Color signalRed ){
		analyseHaar.setColors( backRed, backGreen, gridRed, 
			gridGreen, signalRed );
		analyseDaub2.setColors( backRed, backGreen, gridRed, 
			gridGreen, signalRed );
		synthesisHaar.setColors( backRed, backGreen, gridRed, 
			gridGreen, signalRed );
		synthesisDaub2.setColors( backRed, backGreen, gridRed, 
			gridGreen, signalRed );

	}
		
				
	
	/**********************actionPerformed*************************
	*  ActionEventHandling
  **/
	public void actionPerformed( ActionEvent e ){
		if( e.getActionCommand() == "Haar Wavelet" ){
			head1.setHeadline( e.getActionCommand() );
			plot.setWavelet( new HaarWave() );
			adjust.reset();
		}
		else if ( e.getActionCommand() == "Mexican Hat Wavelet" ){
			head1.setHeadline( e.getActionCommand() );
			plot.setWavelet( new MexicanHat() );
			adjust.reset();
		}
		else if (	e.getActionCommand() == "Morlet Wavelet"){
			head1.setHeadline( e.getActionCommand() );
			plot.setWavelet( new Morlet() );
			adjust.reset();
		}
		else if( e.getActionCommand() == "Haar Filter" ){
			//System.out.println("HaarFilter ActionEvent");
			head2.setHeadline( e.getActionCommand() );
			analyse = analyseHaar;
			synthesis = synthesisHaar;
			button.setPanels( analyse, synthesis );
			transformCenter.removeAll();
			if( button.analysisIsSelected() ){
				transformCenter.add( head2, BorderLayout.NORTH );
				transformCenter.add( analyse, BorderLayout.CENTER );
			}else{
				transformCenter.add( head2, BorderLayout.NORTH );
				transformCenter.add( synthesis, BorderLayout.CENTER );
			}
			validate();
			repaint();
		}
		else if( e.getActionCommand() == "Daubechies-2 Filter" ){
			//System.out.println("Daub2 ActionEvent");
			head2.setHeadline( e.getActionCommand() );
			analyse = analyseDaub2;
			synthesis = synthesisDaub2;
			button.setPanels( analyse, synthesis );
			transformCenter.removeAll();
			if( button.analysisIsSelected() ){
				transformCenter.add( head2, BorderLayout.NORTH );
				transformCenter.add( analyse, BorderLayout.CENTER );
			}else{
				transformCenter.add( head2, BorderLayout.NORTH );
				transformCenter.add( synthesis, BorderLayout.CENTER );
			}
			validate();
			repaint();
		}
		else if( e.getActionCommand() == "slow" ){
			//System.out.println("Menu: SLOW");
			analyseHaar.setSpeed( TransformationPanel.SLOW );
			analyseDaub2.setSpeed( TransformationPanel.SLOW );
			synthesisHaar.setSpeed( TransformationPanel.SLOW );
			synthesisDaub2.setSpeed( TransformationPanel.SLOW );
		}else if( e.getActionCommand() == "medium" ){
			//System.out.println("Menu: MEDIUM");
			analyseHaar.setSpeed( TransformationPanel.MEDIUM );
			analyseDaub2.setSpeed( TransformationPanel.MEDIUM );
			synthesisHaar.setSpeed( TransformationPanel.MEDIUM );
			synthesisDaub2.setSpeed( TransformationPanel.MEDIUM );
		}else if( e.getActionCommand() == "fast" ){
			//System.out.println("Menu: FAST");
			analyseHaar.setSpeed( TransformationPanel.FAST );
			analyseDaub2.setSpeed( TransformationPanel.FAST );
			synthesisHaar.setSpeed( TransformationPanel.FAST );
			synthesisDaub2.setSpeed( TransformationPanel.FAST );
		}else if( e.getActionCommand() == "Wavelets" ){
			tPan.setSelectedIndex(0);
		}else if( e.getActionCommand() == "Transformation" ){
			tPan.setSelectedIndex(1);
		}else if ( e.getActionCommand() == "Analysis" ){
			//the radiobuttons analysis and synthesis of butPan decide
			//which transformationPanel is to be shown
			//System.out.println("analyseButton ActionEvent");
			transformCenter.removeAll();
			transformCenter.add( head2, BorderLayout.NORTH );
			transformCenter.add( analyse, BorderLayout.CENTER );
			validate();
			repaint();
		}else if ( e.getActionCommand() == "Synthesis" ){
			//System.out.println("synthesisButton ActionEvent");
			transformCenter.removeAll();
			transformCenter.add( head2, BorderLayout.NORTH );
			transformCenter.add( synthesis, BorderLayout.CENTER );
			validate();
			repaint();
		}

			


			
	 	
	}
	
	
	
	/**********************stateChanged()**************************
	*	ChangeEventHandling		
	**/	
	public void stateChanged( ChangeEvent e ){
		// if Tab "Wavelets" is not selected,
		// MenuItem choose gets enabled
	  if ( tPan.getSelectedIndex() == tPan.indexOfTab("Wavelets") ){
			menu.setChoose( true );
		} 
		else{
			menu.setChoose( false );
		}
	}// end of stateChanged



}