package transformation;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

/**
*A <em>TransformationPanel</em> is a <code>JPanel</code>
*on which the multiresolution-animation is painted
*@version 26/05/2001
*@author Julia Schneider
*/
public class AnalyseDaub2Panel extends AnalysePanel{
	
	/**
	*standard constructor
	*/
	public AnalyseDaub2Panel(){
		//chosen filter
		filter = new Daub2();
		
		//transformed signals
		transformed = new Grid[3];
		for( int i = 0; i < transformed.length; i++ ){
			transformed[i] = new Grid(signal.getLength());
		}
		
		filter.fold( signal, transformed );
		
		//animated Grids
		animGrid = new AnimatedGrid[3];
		for( int i = 0; i < animGrid.length; i++ ){
			animGrid[i] = new AnimatedGrid( transformed[i], this );
		}
		
		//animated Arrow
		arrow = new AnimatedArrow[16];
		for( int i = 0; i < arrow.length; i++){
			arrow[i] = new AnimatedArrow(this);
		}
		
		//animation speed
		timer = new Timer(MEDIUM, this);
		
		//checking which grid is to be animated
		gridCheckTime = new boolean[3];
		gridCheckTime[0] = true;

	}
	
	
	/** 
	* painting the Component
	*/
	public void paintComponent(Graphics g ){
		super.paintComponent( g );
	//	System.out.println("This is a AnalyseDaub2Panel");
		//drawing animated grids
		if( animateGrid && depth < 3 ){
			drawAnimGrid( g );			
		}
	}//end of PaintComponent()
	
	
	/**
	*this method draws the original signal 
	*and all at the current level of transformation 
	*necessary transformed signals
	*@param g Graphics object of paintComponent(...)
	*/
	public void drawNecessaryGrids( Graphics g ){
		
		//drawing the signal
		signal.drawGrid( g, 0, 0, Color.white );
		for (int i = 0; i < signal.getLength(); i++ ){
			signal.drawValue( g, i, Grid.signalRed);
		}
		g.setColor( Color.black );
		g.drawString( "SIGNAL", 
		width+5, signal.getWritingPosition());
										
		//drawing grids and values of the yet transformed signals
		for( int x = 0; x < depth; x++ ){
			int y = (int) ((x+1)*signal.getGap()+(x+1)*10);
			transformed[x].drawGrid(g , 0, 
				y,	Color.white );

			//drawing a fat red borderline between the values yet transformed and
			//these to be transformed 
			if ( depth > 0 ){
				g.setColor( Color.red );
				int xPos = (int) ( Math.pow( 2, 3-x ) * Grid.getGap() );
				g.drawLine( xPos-1, y-3, xPos-1, y+Grid.getGap()+3);
				g.drawLine( xPos, y-3, xPos, y+Grid.getGap()+3);
				g.drawLine( xPos+1, y-3, xPos+1, y+Grid.getGap()+3);
			}

			g.setColor( Color.black );
			g.setFont( new Font("Serif",Font.BOLD, 14) );
			g.drawString( (x+1) + ".TRANSFORMED", width+5, 
			transformed[x].getWritingPosition());
			g.setFont( font );
			for(int i = 0; i < transformed[x].getLength();i++){
				transformed[x].drawValue(g,i);
			}
		}
			
		//drawing the grid of the signal 
		//which is to be transformed next
		if( depth < 3 ){
			transformed[depth].drawGrid( g, 0, height, Color.white );
			//drawing the yet transformed values
			for( int i = transformed[depth].getBorder()*2;
			i < transformed[depth].getLength(); i++){
				transformed[depth].drawValue( g, i);
			}
			g.setColor( Color.black );
			g.setFont( new Font("Serif",Font.BOLD, 14) );
			g.drawString( (depth+1) +".TRANSFORMED", width+5,
				transformed[depth].getWritingPosition());
			g.setFont( font );
		}
	}//end of drawNecessaryGrids
		
	
	public void transform( Graphics g, int i, int pos, Color color ){
		
		if( filterCheckTime[i] ){			
			//drawing the filter
			//System.out.println("pos = " + pos );
			//System.out.println("Math.pow( 2, 3-depth ) =" + Math.pow( 2, 3-depth ) );
			if( pos < Math.pow( 2, 3-depth ) -1 ){
								
				//drawing the filter
				filter.drawGrid( g, 2*pos*Grid.getGap(),
					(signal.getGap()+10)*(depth+1)+pos*signal.getGap(), 
						color);
				
				for (int j = 0; j < filter.getLength(); j++ ){
					filter.drawValue( g, j, Color.white);
				}
			
				if( depth == 0 ){
					signal.setActive( 2*pos, filter.getLength(), color );
				}else if (depth < 3 ){
					transformed[depth-1].setActive( 2*pos, filter.length(), 
						color);
				}
			}//end of if( i < Math.pow( 2, 3-depth ) )
			else{
				
				//drawing the filter
				for( int q = 0; q < 4; q++ ){
						
					int yPos = (signal.getGap()+10)*(depth+1)+pos*signal.getGap();
					int xPos;
					int position;
					
					if( q < 2 ){
						//drawing the first part of the splitted grid
						position = (int) (Math.pow( 2, 4-depth)+(q-2));
						xPos = position*Grid.getGap();
						
						if( q == 0 ){
							filter.drawSingleGrid( g, xPos, yPos, color, true );
						}else{
							filter.drawSingleGrid( g, xPos, yPos, color, false );
						} 
							
						filter.drawValue( g, q, xPos, yPos );
												
					}else{ //q>=2
						//drawing the second part of the splitted grid
						
						position = q-2;
							
						xPos = position * Grid.getGap();
																	
						filter.drawSingleGrid( g, xPos, yPos, color, false );						
						filter.drawValue( g, q, xPos, yPos );
					}// end of if (q<2)
					
					
					//drawing the active region of the signal to be transformed
					if( depth == 0 ){
						signal.drawSingleActiveRegion( g, xPos, position, color, Color.red );
					}else if (depth < 3){ 
						transformed[depth-1].drawSingleActiveRegion( g, xPos, 
							position, color, Color.red );
					}
					
				}//for q = 0; q < 4
			}
		}//end of if( filterCheckTime[i] )


		if( arrowCheckTime[i] ){
			//drawing an animated arrow
			//System.out.println("arrowCheckTime"+i);
			
			timer.stop();
			if( depth == 0 ){
				
				String text = ""; 
				for ( int q = 0; q < filter.getLength(); q++ ){
					text += signal.get( (2*pos+q) % signal.getLength() )
										+ "*"	+ filter.getRounded(q);
					if( q < filter.getLength()-1 )		
						text +=  " + ";
				}

				arrow[i].drawAnimatedArrow(g, filter.getArrowStart(0),
					transformed[depth].getArrowStop(i), text);
			}else if ( depth < 3 ){
				
				String text = ""; 
				for ( int q = 0; q < filter.getLength(); q++ ){
					text += transformed[depth-1].getRounded( 
									(2*pos+q)%transformed[depth-1].getLength() )
							+ "*"	+ filter.getRounded(q);
					if( q < filter.getLength()-1 )		
						text +=  " + ";
				}
					
				arrow[i].drawAnimatedArrow( g, filter.getArrowStart(0),
					transformed[depth].getArrowStop(i), text );
			}
			
			if( arrow[i].finished() ){
				arrowCheckTime[i] = false;
				transformedCheckTime[i] = true;
				//System.out.println( "counter = " + counter );
				//System.out.println( "transformedCheckTime" +i+" "
				//+ transformedCheckTime[i] );
				timer.restart();
			}
		}//end of if (depth < 3)
		
		if( transformedCheckTime[i] ){
			//drawing the result into the transformed Signal
			transformed[depth].drawValue(g, i, color);
			filterCheckTime[i] = false;
		}
	}//end of daub2Transform()
	
	
	/**
	* Handling of the ActionEvents
	*/
	public void actionPerformed( ActionEvent e ){
	
		if( e.getSource() == timer ){
			System.out.println("C O U N T E R " + counter );
		}
		if( depth <= 2 ){
			if( counter < 16/Math.pow(2, depth) ){
			
				if(counter == 0 ){
					filter.setValues( filter.getApproximation() );
					System.out.println( "ApproximationFilter" );
				}else	if( counter == 8/Math.pow(2, depth) ){
					//detail-time
					filter.setValues( filter.getDetail() );
					System.out.println( "DetailFilter" );
				}
				//approximation time
				filterCheckTime[counter] = true;
				//System.out.println( "filterCheckTime" +counter+" "
				//+ filterCheckTime[counter] );
				arrowCheckTime[counter] = true;
				//System.out.println( "arrowCheckTime" +counter+" "
				//+ arrowCheckTime[counter] );
			}
			else if( counter == 16/Math.pow(2, depth) ){
				for( int i = 0; i < signal.getLength(); i++){
					transformedCheckTime[i] = false;
					arrow[i].reset();
					if( i < animGrid.length ){
						animGrid[i].reset();
					}
				}
				startTransform = false;
				animateGrid = true;
				counter = -1;
			}
			counter++;
		}else{ //depth >= 3
			timer.stop();
			startTransform = false;
		}
		repaint();
	}//end of actionPerformed						


}//end of class
