package transformation;
import java.awt.event.*;
import javax.swing.*;
import java.awt.*;

/**
* class for drawing animated arrows. Start by placing method
* drawAnimatedArrow() in the paintComponent() method of JPanel 
* panel. Now every millisecond*speed, Timer t fires an ActionEvent
* which causes one single arrow to be painted.
*/
public class AnimatedArrow implements ActionListener{
	private Point p1, p2; //starting- and endposition of the arrow
	private JPanel panel; //JPanel on which to draw
	private Timer t;
	private Font arrowFont, stringFont;
	private double gradient; //gradient of the arrow
	private int counter, pace, speed;
	private boolean firstTime, finished;
	public static final int SLOW=20, MEDIUM=10, FAST=5;
		
	/**
	*@param p TransformationPanel where to draw on
	*/
	AnimatedArrow( JPanel p ){
		speed = 10;
		pace = 10;
		counter = 10;
		arrowFont = new Font("Serif",Font.BOLD, 16);
		stringFont = new Font("Serif",Font.BOLD, 12);
		panel = p;
		firstTime = true;
		t = new Timer( speed, this );
 }
	
	/**
	* draws an arrow from point p1 to point p
	*@param g Graphics object of the calling JPanel
	*@param p ending Point of the arrow
	*@param s String which is painted in front of the arrowhead
	*/
	public void drawArrow(Graphics g, Point p, String s){
		//System.out.println("draw one AnimatedArrow");

		g.setFont(arrowFont);
		g.drawLine( (int) p1.getX(), (int) p1.getY(), 
			(int) p.getX(), (int) p.getY()-8 );
		
		// painting the arrowhead
		Polygon pol = new Polygon();
		pol.addPoint( (int) p.getX(), (int) p.getY());
		pol.addPoint( (int) p.getX()-8, (int) p.getY()-8 );
		pol.addPoint( (int) p.getX()+8, (int) p.getY()-8 );

		g.fillPolygon(pol);
		FontMetrics fm = g.getFontMetrics();
		g.setFont( stringFont );
		
		if( p2.getY()-p.getY() > fm.getHeight() ){
			g.drawString( s,(int) (p.getX()-fm.stringWidth(s)/3), 
								(int) (p.getY()+fm.getHeight()) );
		}
	}
	
	public void reset(){
		firstTime=true;
		finished = false;
		counter = 10;
	}
	
	/**
	* starts drawing an animated Arrow from Point p1
	* to Point p2;
	* is to be placed in the paintComponent method of
	* the calling JPanel
	*@param g Graphics object of the calling JPanel
	*@param p1 starting Point
	*@param p2 ending Point
	*@param s String which is to be painted in front of the arrowhead
	*/
	public void drawAnimatedArrow(Graphics g, Point p1, 
			Point p2, String s){
		if( firstTime ){
				//System.out.println("first Time AnimatedArrow");
			this.p1 = p1;
			this.p2 = p2;
			gradient = (p2.getX()-p1.getX())/(p2.getY()-p1.getY());
			firstTime = false;
			t.start();
		}
		
		int y =(int) p1.getY()+counter;
		int x =(int) (p1.getX() + gradient*( y-p1.getY() ) );
		
		drawArrow(g, new Point(x,y), s);
	}

	public boolean finished(){
		return finished;
	}

	public boolean isRunning(){
		return t.isRunning();
	}
	
	public void stopTimer(){
		t.stop();
	}
	
	public void setSpeed( int speed ){
		
		this.speed = speed;
		
		if (speed==SLOW){
			t.setDelay( speed );
			pace = 2;
			//System.out.println( "arrowSlow");
		}
		else if( speed == MEDIUM){
				t.setDelay( speed );
				pace = 10;
				//System.out.println( "arrowMedium");
		}else if( speed == FAST){
				t.setDelay( speed );
				pace = 25;
				//System.out.println( "arrowFast");
		}
		
	}

				
	public void restartTimer(){
	  if( !finished ){
	  	t.restart();
	  }
	}
	/**
	* handels the ActionEvents that are fired every
	* milli-second by Timer t
	*/
	public void actionPerformed( ActionEvent e ){
		if( p1.getY()+counter <=  p2.getY()-pace ){
			//System.out.println("counter "+counter);
			panel.repaint();
			counter += pace;
		}else{
			//System.out.println( "finished" );
			finished = true;
			t.stop();	
			panel.repaint();
		}
	}
	
}