package transformation;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

public class ButtonPanel extends JPanel
			implements ActionListener{
	
	private JFrame parent;
	private AnalysePanel analysisPan ;
	private SynthesisPanel synthesisPan;
	private JButton start, reset, pause;
	private JRadioButton analysis, synthesis;
	private JButton detail;
	private int arrowCounter, gridCounter;
	private boolean pausePressed;
	private Color color;
	

	public ButtonPanel( AnalysePanel a, SynthesisPanel s, JFrame f ){
		parent = f;
		analysisPan = a;		
		synthesisPan = s;

		setLayout( new GridLayout( 4, 1) );
		
		start = new JButton("Start");
		reset = new JButton("Reset");
		pause = new JButton("Pause");

		analysis = new JRadioButton("Analysis", true );
		synthesis = new JRadioButton("Synthesis");
		detail = new JButton( "Details" );
		
		ButtonGroup bg= new ButtonGroup();
		bg.add( analysis );
		bg.add( synthesis );
		
		Box b = Box.createVerticalBox();
		b.add( Box.createVerticalGlue() );
		b.add( detail );
		b.add( Box.createVerticalGlue() );
		b.add( analysis );
		b.add( synthesis );
		b.add( Box.createVerticalGlue() );
	
		
		add( b );
		add(start);
		add(pause);
		add(reset);
	
		
		start.addActionListener( this );
		reset.addActionListener( this );
		pause.addActionListener( this );
		
		analysis.addActionListener( (ActionListener) parent );
		synthesis.addActionListener( (ActionListener) parent );
		
		detail.addActionListener( this );
		
		start.setMnemonic(KeyEvent.VK_S);
		reset.setMnemonic(KeyEvent.VK_R);
		pause.setMnemonic(KeyEvent.VK_P);
			
		start.addKeyListener( new KeyListener(){
			public void keyTyped( KeyEvent e ){
				actionPerformed( new ActionEvent(start, 0, "start") );
			}
			public void keyPressed( KeyEvent e ){}
			public void keyReleased( KeyEvent e ){}
			});
		reset.addKeyListener( new KeyListener(){
			public void keyTyped( KeyEvent e ){
				actionPerformed( new ActionEvent(reset, 1, "reset") );
			}
			public void keyPressed( KeyEvent e ){}
			public void keyReleased( KeyEvent e ){}
			});
		pause.addKeyListener( new KeyListener(){
			public void keyTyped( KeyEvent e ){
				actionPerformed( new ActionEvent( pause, 2, "pause" ) );
			}
			public void keyPressed( KeyEvent e ){}
			public void keyReleased( KeyEvent e ){}
			});
	
		/*color = Color.white;
			
		Border b = BorderFactory.createLineBorder( color,5);
		start.setBorder( b );
		reset.setBorder( b );
		pause.setBorder( b );
		synthesis.setBorder( b );
		*/
		
		pause.setEnabled( false );
		reset.setEnabled( false );
			
		//default value, means that no arrow is active
		arrowCounter = 999;
		gridCounter = 999;
		
	}
	
	
	
	 
	public void actionPerformed( ActionEvent e ){
		if ( e.getSource() == start ){
			start.setEnabled( false );
			pause.setEnabled( true );
			reset.setEnabled( true );
			analysis.setEnabled( false );
			synthesis.setEnabled( false );
			
			TransformationPanel dummy = null;
			if (analysis.isSelected() ){
				dummy = analysisPan;
			}else if (synthesis.isSelected() ){
				dummy = synthesisPan;
			}
			
			dummy.startTransform(true);
			if( pausePressed ){
					dummy.restartTimer( 
						new int[]{ arrowCounter, gridCounter} );
					pausePressed = false;
			}else{
				dummy.startTimer();
			}			
			
		}else if ( e.getSource() == pause ){
			int[] dummy = null;
			if ( analysis.isSelected() ){
				dummy = analysisPan.stopTimer();
			}else if ( synthesis.isSelected() ){
				dummy = synthesisPan.stopTimer();
			}
			arrowCounter =	dummy[0];
			gridCounter = dummy[1];
			pause.setEnabled( false );
			start.setEnabled( true );	
			analysis.setEnabled( true );
			synthesis.setEnabled( true );
			pausePressed=true;
		}else if ( e.getSource() == reset ){
			if( analysis.isSelected() ){
				analysisPan.reset();
			}else if ( synthesis.isSelected() ){
				synthesisPan.reset();
			}
			pause.setEnabled( false );
			start.setEnabled(true);
			reset.setEnabled( false );
		}else if ( e.getSource() == detail ){
			String message = null;

			if (analysisPan instanceof AnalyseHaarPanel ){
				message = "THE CURRENT FILTERBANK IS:\n\n"					
					+"Analysis Filter:\n"
					+"   Lowpass:   0.5   0.5\n"
					+"   Highpass:  0.5  -0.5\n\n\n"
					+"Synthesis Filter:\n"
					+"   Lowpass:   1   1\n"
					+"   Highpass:  1  -1\n\n\n";
			}else if (analysisPan instanceof AnalyseDaub2Panel ){
				message = "THE CURRENT FILTERBANK IS:\n\n"					
					+"Analysis Filter:\n"
					+"   Lowpass:    0.482963   0.836516  0.224144  -0.12941\n"
					+"   Highpass:  -0.12941   -0.224144  0.836516  -0.482963\n\n"
					+"Synthesis Filter:\n"
					+"   Lowpass:    0.224144   0.836516  0.482963  -0.12941\n"
					+"   Highpass:  -0.12941   -0.482963  0.836516  -0.224144\n\n";
			}
				
			JOptionPane.showMessageDialog( parent, message, 
				"Detail Information", JOptionPane.INFORMATION_MESSAGE); 
			
		}
	}
		
	public void setPanels( AnalysePanel a, SynthesisPanel s ){
		analysisPan = a;
		synthesisPan = s;
	}
	
	public boolean analysisIsSelected(){
		return analysis.isSelected();
	}
	
	public boolean synthesisIsSelected(){
		return synthesis.isSelected();
	}
	
	public void setFinishedSituation(){
		analysis.setEnabled( true );
		synthesis.setEnabled( true );
		pause.setEnabled( false );
	}
}