package transformation;
import java.awt.*;

public class Grid{
	private static int gap;	//space between two vertical lines
	private double[] values, roundedValues;
	protected int originX, originY, marginT, activeStart, activeLength,
		border;
	protected Font font;
	protected FontMetrics fm;
	private Color active;
	static Color red = new Color(181,36,57), signalRed = Color.red,
		green = new Color(0,128,0);
		
	/**
	*standard constructor
	*/
	public Grid(){
		active = Color.white;
		font = new Font("Serif",Font.BOLD, 16);
		border = 16;
	}
	
	/**
	*creates a Grid-object with length number of values
	*/
	public Grid( int length ){
		this();
		values = new double[length];
		roundedValues = new double[ length ];
	}
		
	/**
	*creates a Grid-object with values as values
	*/
	public Grid(double[] values){
		this( values.length );
		this.values = values;		
	}
	
	
	public void roundValues(){
		for( int i = 0; i < values.length; i++ ){
			roundedValues[i] = (double) (Math.round( 100 * values[i] ) )/ 100;
		}
	}
	
	/**
	*creates a Grid-object with values as values
	*and a gap
	
	public Grid( double[] values, int gap ){
		this( values );
		this.gap = gap;	
	}*/

	
	
	
	/**
	*draws a grid consisting of a single square. 
	*@param g Graphics object for graphical context
	*@param x upper left x-position of the square
	*@param y upper left y-position of the square
	*@param fill background color of the grid
	*/
	public void drawSingleGrid( Graphics g, int x, int y,	Color fill, boolean firstSquare ){
			
		fm = g.getFontMetrics( font);
		g.setFont( font );
		
		//only change the origin if it's the first square of the grid
		if( firstSquare ){
			originX = x;
			originY = y;			
		}

		
		//filling the grid
		g.setColor( fill );
	  g.fillRect( x, y, gap, gap );
	  
	  //drawing the rectangle
		g.setColor( Color.black );
		g.drawRect( x, y, gap, gap );
		
	}


	
	/**
	*draws a grid at (x,y)
	*@param g Graphics object for graphical context
	*@param x upper left x-position of the grid (see originX)
	*@param y upper left y-position of the grid (see originY)
	*@param fill background color of the grid
	*/
	public void drawGrid( Graphics g, int x, int y, 
		Color fill){
		
		fm = g.getFontMetrics( font);
		g.setFont( font );
							
		originX = x;
		originY = y;			
		
		//filling the grid
		g.setColor( fill );
	  g.fillRect( originX, originY, gap* values.length, gap );
				
	  //filling the active region
	  g.setColor( active );
	  g.fillRect( originX+activeStart*gap, originY,
	  						gap* activeLength, gap );
	  active = Color.white;
	  
	  //drawing the rectangle
		g.setColor( Color.black );
		g.drawRect( originX, originY, gap* values.length, gap );
		
	  //painting the vertical lines
		for( int i= 0; i < values.length; i++){
			g.drawLine( originX+i*gap, originY,
				originX+i*gap, originY+gap);
		}
	}	//end of drawGrid()
	
	
	/**
	*draws a single value in to a yet painted grid.
	*@param g Graphics object for the graphics context
	*@param pos adds values[pos] at position pos in the grid
	*@param color of the writing
	*@see drawGrid()
	*/
	public void drawValue( Graphics g, int pos, Color color ){
		g.setFont( font );
	
		//margin to the left/top side of a square
		int marginL = 
			(gap 	- fm.stringWidth( String.valueOf(roundedValues[ pos ])) )/ 2;
		marginT = (gap - fm.getHeight())/ 2;
	
		//x-, y-Position of a given value
		int x = originX + pos*gap + marginL;
		int y = originY + marginT + fm.getAscent();
			
		g.setColor( color );
		
		//draw rounded values at position pos
		g.drawString( String.valueOf( roundedValues[ pos ]), x, y);
			
		g.setColor( Color.black );
	}
	
	
		
	
	/**
	*draws a colored single value into a yet painted grid.
	*@param g Graphics object for the graphics context
	*@param pos adds values[pos] at position pos in the grid
	*@see drawValue(Graphics, int, Color )
	*/
	public void drawValue( Graphics g, int pos ){
		if( pos < border ){
				drawValue( g, pos, signalRed );
		}else{
				drawValue( g, pos, green );
		}
	}
	
	
		/**
	*draws a single, white value in to a yet painted grid.
	*@param g Graphics object for the graphics context
	*@param pos value values[pos] is painted
	*@param x X-Position of the value to be painted
	*@param y Y-Position of the value to be painted
	*/
	public void drawValue( Graphics g, int pos, int x, int y){
			
		//margin to the left/top side of a square
		int marginL = 
			(gap 	- fm.stringWidth( String.valueOf(roundedValues[ pos ])) )/ 2;
		marginT = (gap - fm.getHeight())/ 2;
	
		//x-, y-Position of a given value
		x += marginL;
		y += marginT + fm.getAscent();
			
		g.setFont( font );
		g.setColor( Color.white );
		
		//draw rounded values at position pos
		g.drawString( String.valueOf( roundedValues[ pos ]), x, y);
			
		g.setColor( Color.black );
	}
	
	/**
	*draws the active region containing of only one square
	*@param g Graphics object for graphical context
	*@param x upper left x-position of the active square
	*@param active background color of the square
	*@param valueColor color of the value
	*/
	public void drawSingleActiveRegion( Graphics g, int x, int pos,
		Color active, Color valueColor){
		
		//painting the background
		g.setColor( active );
	  g.fillRect( x, originY, gap, gap );
	  
	  //painting the rectangle
	  g.setColor( Color.black );
	  g.drawRect( x, originY, gap, gap );
	  
	  drawValue( g, pos, valueColor );
	  
	}
	
	
	/**
	*returns the point where to attach the starting point of an
	*arrow
	*@param pos position of the grid where to attach the arrow
	*@return starting position of an arrow
	*/
	public Point getArrowStart( int pos){
		return new Point( originX+gap*pos, originY+ gap);
	}
	
	/**
	*returns the point where to attach the ending point of an
	*arrow
	*@param pos position of the grid where to attach the arrow
	*@return ending position of an arrow
	*/
	public Point getArrowStop( int pos ){
		return new Point( originX+pos*gap+gap/2, originY);
	}
	
	/** 
	*returns the y-WritingPosition of the values
	*/
	public int getWritingPosition(){
		return originY+fm.getAscent();
	}
	
	/** 
	*sets the active region of the grid to the 
	*active-background-color color
	*@param start starting position of the active region
	*@param length length of the active region
	*/
	public void setActive(int start, int length, Color color ){
		activeStart = start;
		activeLength = length;
		active = color;
	}
		
	public static void setColors(  Color gridRed, 
		Color gridGreen, Color signal ){
		red = gridRed;
		green = gridGreen;
		signalRed  = signal;
	}
	
	public double get( int pos ){
		return values[ pos ];
	}
	
	public double getRounded( int pos ){
		return roundedValues[ pos ];
	}
	
	public void set( int pos, double x ){
		values[pos] = x;
	}
	
	public void setValues( double[] values ){
		this.values = values;
		roundedValues = new double[values.length];
		roundValues();
	}
	
	public double[] getValues(){
		return (double[]) values.clone();
	}
	
	public double[] getRoundedValues(){
		return roundedValues;
	}
	
	public int getLength(){
		return values.length;
	}
	
	public void setGap( int gap ){
		this.gap = gap;
	}
	
	public static int getGap(){
		return gap;
	}
	
	public void setBorder( int b){
		border = b;
	}
	
	public int getBorder(){
		return border;
	}
			
	public Point origin(){
		return new Point( originX, originY );
	}
	
	public int originY(){
		return originY;
	}
	
	public void setOriginX( int x ){
		originX = x;
	}

}