package transformation;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
*A <em>SynthesisDaub2Panel</em> is a <code>SynthesisPanel</code>
*on which the synthesis-animation with the Daubechies2-Filter is painted
*@version 26/05/2001
*@author Julia Schneider
*/
public class SynthesisDaub2Panel extends SynthesisPanel{
	
	/**
	*standard constructor
	*/
	public SynthesisDaub2Panel(){
	
		//chosen filter
		filter = new Daub2();
		
		//transformed signals
		transformed = new Grid[3];
		for( int i = 0; i < transformed.length; i++ ){
			transformed[i] = new Grid(signal.getLength());
		}
		filter.fold( signal, transformed );
		
		//animated Grids
		animGrid = new AnimatedGrid[3];
		animGrid[2] = new AnimatedGrid( signal, this );
		for( int i = 0; i < animGrid.length-1; i++ ){
			animGrid[i] = new AnimatedGrid( transformed[1-i], this );
		}			

		//animated Arrow
		arrow = new AnimatedArrow[16];
		for( int i = 0; i < arrow.length; i++){
			arrow[i] = new AnimatedArrow(this);
		}
		
		//animation speed
		timer = new Timer(MEDIUM, this);
		
		//checking which grid is to be animated
		gridCheckTime = new boolean[3];
		gridCheckTime[0] = true;
		
	
	}//end of standard constructor
		
	/** 
	* painting the Component
	*/
	public void paintComponent(Graphics g ){
		super.paintComponent( g );
		//System.out.println("This is a SynthesisDaub2Panel");
		
		//drawing animated grids
		if( animateGrid && depth < 3 ){
			//System.out.println("animateGrid");
			drawAnimGrid( g );
		}
	}//end of PaintComponent()
	

	/**
	*this method draws the starting signal transformed[3]
	*and all at the current level of transformation 
	*necessary synthesized signals
	*@param g Graphics object of paintComponent(...)
	*/
	public void drawNecessaryGrids( Graphics g ){
		
		g.setFont( font );
		
		//drawing the starting signal
		transformed[2].drawGrid( g, 0, 0, Color.white );
		for (int i = 0; i < transformed[2].getLength(); i++ ){
			transformed[2].drawValue( g, i );
		}
		
		//drawing a fat red line
		int bord = transformed[1].getBorder();
		int xP = (int) ( bord * Grid.getGap() );
		int yP = transformed[2].originY();
		g.setColor( Color.red );
		g.drawLine( xP-1, yP-3, xP-1, yP+Grid.getGap()+3);
		g.drawLine( xP, yP-3, xP, yP+Grid.getGap()+3);
		g.drawLine( xP+1, yP-3, xP+1, yP+Grid.getGap()+3);

		//painting the Label
		g.setColor( Color.black );
		g.setFont( new Font("Serif",Font.BOLD, 14) );
		g.drawString( "3.TRANSFORMED", 
			width+5, transformed[2].getWritingPosition());
										
		//drawing grids and values of the yet transformed signals
		Grid dummy = null;
		for( int x = 0; x < depth; x++ ){
			int y = (int) ((x+1)*signal.getGap()+(x+1)*10);
			if ( x < 2 ){
				dummy = transformed[1-x];
			}else if ( x == 2 ){
				dummy = signal;
			}
			
			dummy.drawGrid(g , 0, y,	Color.white );

			//drawing a fat red borderline between the values yet transformed 
			//and these to be transformed 
			int b = dummy.getBorder();				
			int xPos = (int) ( b * 2 * Grid.getGap() );
				
			g.setColor( Color.red );
				
			g.drawLine( xPos-1, y-3, xPos-1, y+Grid.getGap()+3);
			g.drawLine( xPos, y-3, xPos, y+Grid.getGap()+3);
			g.drawLine( xPos+1, y-3, xPos+1, y+Grid.getGap()+3);
			
			//drawing the label
			g.setColor( Color.black );
			g.setFont( new Font("Serif",Font.BOLD, 14) );

			if( x < 2 ){
				g.drawString( (2-x) + ".TRANSFORMED", width+5,
					dummy.getWritingPosition());
			}else if ( x == 2 ){
				g.drawString( "SIGNAL", width+5,
					dummy.getWritingPosition());			
			}
				
			//drawing the values		
			g.setFont( font );
			for(int i = 0; i < dummy.getLength();i++){
					dummy.drawValue(g,i);
			}

		}//end of for (x = 0; x< depth; x++)
		
		//drawing the grid of the signal 
		//which is to be transformed next
		if( depth < 3 ){
						
			if( depth < 2 ){
				dummy = transformed[1-depth];
			}else if ( depth == 2){
				dummy = signal;
			}
			
			//drawing the grid
			dummy.drawGrid( g, 0, height, Color.white );
			
			//drawing the green values
			for( int i = dummy.getBorder();	i < dummy.getLength(); i++){
				dummy.drawValue( g, i);
			}
			
			//drawing a fat red line
			int b = dummy.getBorder();
			int xPos = (int) ( b * Grid.getGap() );
			int y = dummy.originY();
			g.setColor( Color.red );
			g.drawLine( xPos-1, y-3, xPos-1, y+Grid.getGap()+3);
			g.drawLine( xPos, y-3, xPos, y+Grid.getGap()+3);
			g.drawLine( xPos+1, y-3, xPos+1, y+Grid.getGap()+3);
			
			//drawing the Label
			g.setColor( Color.black );
			g.setFont( new Font("Serif",Font.BOLD, 14) );
			
			if( depth < 2 ){
				g.drawString( (2-depth) +".TRANSFORMED", width+5,
					dummy.getWritingPosition());
			}else if ( depth == 2 ){
				g.drawString( "SIGNAL", width+5, dummy.getWritingPosition());
			}
			
			g.setFont( font );
		}

	}
	
	/**
	*this method prepares the animation of the
	*synthesis by starting the Timer-object timer
	*which now will fire ActionEvents every speed*milliseconds
	*causing the synthesis to develop
	*@param g Graphics object of paintComponent(...)
	*@see actionPerformed(ActionEvent e)
	*/
	public void startTransformation( Graphics g ){
		super.startTransformation( g );
		
		Color color = null;
		if( filter.actualFilter().equals("synthesis1") ){
			color = Grid.red;
		}else if( filter.actualFilter().equals("synthesis2")){
			color = Grid.green;
		}
		
		for ( int i = 0; i < 16/Math.pow(2, 2-depth); i++ ){
			transform( g, i,  color );
		}

	}

	
	public void transform( Graphics g, int i, Color color ){
				
		int pos= 0;//position of the current filter to be drawn
			
		if( filterCheckTime[i] ){
			//System.out.println( "filterCheckTime["+i+"]=" + filterCheckTime[i]);
		
		//determination of pos
			if( filter.actualFilter().equals("synthesis1") ){
				pos = i;
				//System.out.println( "i=pos="+i);				
			}else if( filter.actualFilter().equals("synthesis2")){
				int threshold=(int) ( 8/Math.pow(2, 2-depth) );	
				pos = i-threshold;
				//System.out.println( "i="+i);
				//System.out.println( "(int) (8/Math.pow(2, 2-depth) ="+ threshold);
				//System.out.println( "pos=" + pos );
			}
			
			//drawing the yet calculated values
			Grid dummy = null;
		
			if ( depth < 2 ){
				dummy = transformed[1-depth];
			}else if ( depth == 2 ){
				dummy = signal;
			}
			
			if( filter.actualFilter().equals("synthesis1") ){
				for ( int x = 0; x < pos; x++){
					dummy.drawValue(g, x*2, Color.red );
				}
			}else if( filter.actualFilter().equals("synthesis2") ){
					
				for( int x = 0; x < dummy.getBorder(); x +=2){
					dummy.drawValue(g, x, Color.red );
				}
				
				for ( int x = 0; x < pos; x++){
					dummy.drawValue(g, x*2+1, Color.red );
				}
			}// end of drawing the yet calculated values	
			
	
			//drawing the filter
			int[] position = new int[filter.getLength()];			
			
			if ( pos == 0 ){
				position[0] = pos+transformed[2-depth].getBorder()-1;
				position[1] = 2*transformed[2-depth].getBorder()-1;
				position[2] = pos;
				position[3] = pos+transformed[2-depth].getBorder();
			}else if ( pos >= 1 ){
				position[0] = pos-1;
				position[1] = pos-1+transformed[2-depth].getBorder();
				position[2] = pos;
				position[3] = pos+transformed[2-depth].getBorder();
			}
			
			for( int j = 0; j < filter.getLength(); j++ ){
							
				int xPos= position[j] * Grid.getGap();
				int yPos = (Grid.getGap()+10)*(depth+1)+pos*Grid.getGap()/2;
				
				if ( j == 0 ){
					//drawing one square with its belonging value
					filter.drawSingleGrid( g, xPos, yPos, color, true );
					//drawing the active region of the signal to be synthesized
					transformed[2-depth].drawSingleActiveRegion( g, xPos,
						position[j], color, Color.red );
				}else if ( j == 1 || j == 3 ){
					//drawing one square with its belonging value
					filter.drawSingleGrid( g, xPos, yPos, color, false );
					//drawing the active region of the signal to be synthesized
					transformed[2-depth].drawSingleActiveRegion( g, xPos,
						position[j], color, Color.green );
				}else if ( j == 2 ){
					//drawing one square with its belonging value
					filter.drawSingleGrid( g, xPos, yPos, color, false );
					//drawing the active region of the signal to be synthesized
					transformed[2-depth].drawSingleActiveRegion( g, xPos,
						position[j], color, Color.red );
				}

				
				filter.drawValue( g, j, xPos, yPos );
					
			}//end of drawing the filter for( j = 0; j < filter.length;j++)
	

		
			//drawing an animated arrow
			if( arrowCheckTime[i] ){
					
				//System.out.println("arrowCheckTime"+i);
				//System.out.println("pos="+pos);
			
				timer.stop();
			
			
				//System.out.println("transformed[2-depth].getRounded(pos)="
					//+transformed[2-depth].getRounded(pos) );
				
				//System.out.println("transformed[2-depth].getRounded( pos+transformed[3-depth].getBorder())=" 
					//+ transformed[2-depth].getRounded(pos+transformed[2-depth].getBorder()));
				
				String text = "";
				for ( int j = 0; j < filter.getLength()-1;j++){
					text += transformed[2-depth].getRounded(position[j]) + "*" 
											+ filter.getRounded(j) + " + " ;
				}
				text += transformed[2-depth].getRounded(position[3]) + "*" 
											+ filter.getRounded(3);																						
				//System.out.println("text="+text);
			
				if( filter.actualFilter().equals("synthesis1") ){
					if( depth < 2 ){
						arrow[i].drawAnimatedArrow(g, filter.getArrowStart(0),
							transformed[1-depth].getArrowStop(2*pos), text);
					}else if ( depth == 2 ){
						arrow[i].drawAnimatedArrow(g, filter.getArrowStart(0),
							signal.getArrowStop(2*pos), text);
					}
				}else if( filter.actualFilter().equals("synthesis2") ){
				
					if( depth < 2 ){
						arrow[i].drawAnimatedArrow(g, filter.getArrowStart(0),
							transformed[1-depth].getArrowStop(2*pos+1), text);
					}else if ( depth == 2 ){
						arrow[i].drawAnimatedArrow(g, filter.getArrowStart(0),
							signal.getArrowStop(2*pos+1), text);
					}
				}//end of if (filter.actualFilter().equals(...)	

				if( arrow[i].finished() ){
					arrowCheckTime[i] = false;
					transformedCheckTime[i]= true;
					timer.restart();				
				}
			}//end of drawing an animated arrow
		
		
			//drawing the result into the transformed Signal
			if( transformedCheckTime[i] ){
				//System.out.println("transformedCheckTime["+i+"]=" + transformedCheckTime[i] );
				//System.out.println( "pos=" + pos );
	
				if( filter.actualFilter().equals("synthesis1") ){
					if (depth < 2 ){
						transformed[1-depth].drawValue(g, 2*pos, Color.red);
						//System.out.println( "2*pos=" + 2*pos );
					}else if ( depth == 2 ){
						signal.drawValue( g, 2*pos, Color.red );
					}
				}else if( filter.actualFilter().equals("synthesis2") ){
					if (depth <  2 ){
						//System.out.println( "2*pos+1=" + (2*pos+1) );
						transformed[1-depth].drawValue(g, 2*pos+1, Color.red);
					}else if ( depth == 2 ){
						signal.drawValue( g, 2*pos+1, Color.red );
					}
				}
			
				filterCheckTime[i] = false;
			}//endOf if( transformedCheckTime[i] )
		}//end of if ( filterCheckTime[i] )
		
	}
	

	/**
	* Handling of the ActionEvents
	*/
	public void actionPerformed( ActionEvent e ){
	
		if( e.getSource() == timer ){
			System.out.println("C O U N T E R " + counter );
		}
		if( depth <= 2 ){
			if( counter < 16/Math.pow(2, 2-depth) ){
			
				if(counter == 0 ){
					filter.setValues( filter.getSynthesis1() );
					System.out.println( "Synthesis1Filter" );					
				}else	if( counter == 8/Math.pow(2, 2-depth) ){
					//detail-time
					filter.setValues( filter.getSynthesis2() );
					System.out.println( "Synthesis2Filter" );
				}
				//synthesis1 time
				filterCheckTime[counter] = true;
				//System.out.println( "filterCheckTime" +counter+" "
				//	+ filterCheckTime[counter] );
				arrowCheckTime[counter] = true;
				//System.out.println( "arrowCheckTime" +counter+" "
				//	+ arrowCheckTime[counter] );
			}
			else if( counter == 16/Math.pow(2, 2-depth) ){
				for( int i = 0; i < signal.getLength(); i++){
					transformedCheckTime[i] = false;
					arrow[i].reset();
					if( i < animGrid.length ){
						animGrid[i].reset();
					}
				}
				startTransform = false;
				animateGrid = true;
				counter = -1;
			}
			counter++;
		}else{ //depth >= 3
			timer.stop();
			startTransform= false;
		}
		repaint();
	}//end of actionPerformed
	
}//end of class