package transformation;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

/**
*A <em>TransformationPanel</em> is a <code>JPanel</code>
*on which the multiresolution-animation is painted
*@version 26/05/2001
*@author Julia Schneider
*/
public abstract class TransformationPanel extends JPanel 
	implements ActionListener{
	protected Filter filter;
	protected Grid signal;
	protected Grid[] transformed;
	protected AnimatedGrid[] animGrid;
	protected AnimatedArrow[] arrow;
	protected Timer timer;
	protected Font font;
	protected FontMetrics fm;
	protected Color backRed, backGreen;
	protected boolean startTransform, animateGrid;
	protected boolean[] filterCheckTime, arrowCheckTime, 
		transformedCheckTime, gridCheckTime;
	protected int width, height, counter, depth;
	public static final int SLOW = 2000, MEDIUM = 1000, FAST = 1;
	
	
	public TransformationPanel(){
		
		//signal to be transformed
		signal = new Grid( new double[]
				{ 1, 2, 2, 3, 2, 3, 4, 1, 1, 2, 2, 1, 1, 3, 2, 4} );
				
		//checking what is when to be painted
		filterCheckTime = new boolean[16];
		arrowCheckTime = new boolean[16];
		transformedCheckTime = new boolean[16];
		
		//style
		font = new Font("Serif",Font.BOLD, 16);
	
		//background colors of the animation
		backRed = new Color( 255, 189, 189 );
		backGreen = new Color( 202, 255, 202 );
		setBackground( backRed );
		
	}//end of default constructor
		
	
	public abstract void drawNecessaryGrids( Graphics g );
	
	/**
	*draws an animated Grid
	*/
	public abstract void drawAnimGrid( Graphics g );

	
	public abstract void startTransformation( Graphics g);
	
	
	
	/** 
	* painting the Component
	*/
	public void paintComponent(Graphics g ){
		super.paintComponent( g );
		
		g.setFont( font);
		fm = g.getFontMetrics( font);

		/*width/height of the grid area		
			160 = 2*side-margin (30) + space for signature 
			at the right side
			(60+width/signal.getLength)= 2*margin(=30)+signal.gap
		*/
		width = getSize().width - 160;
		height = getSize().height - (60+width/signal.getLength());
		g.translate( 30, 30 ); //new origin
		//space between to vertical or to horizontal lines of a grid
		signal.setGap( width/signal.getLength() );
		
		//drawing all at the moment necessary grids
		drawNecessaryGrids( g );
				
		//starting the transformation
		if( startTransform ){
			startTransformation(g);
		}
		
	}//end of PaintComponent()

	
	/**
	*this method resets all variables to its
	*standard value causing the beginning state
	*/
	public void reset(){
		timer.stop();
		for( int i = 0; i < arrow.length; i++ ){
			if( arrow[i].isRunning() ){
				arrow[i].stopTimer();
			}
			arrow[i].reset();
			filterCheckTime[i] = false;
			arrowCheckTime[i] = false;
			transformedCheckTime[i] = false;
		}
		for( int i = 0; i < animGrid.length; i++ ){
			if( animGrid[i].isRunning() ){
				animGrid[i].stopTimer();
			}
			gridCheckTime[i] = false;
			animGrid[i].reset();
		}
		counter = 0;
		depth = 0;
		gridCheckTime[0] = true;
		startTransform = false;
		animateGrid = false;
		signal.setActive(0,0,Color.white );
		transformed[0].setActive(0,0,Color.white );
		//setBackground( Color.white );
		repaint();
	}
	
	public void startTimer(){
		timer.start();
	}
	
	
	public int[] stopTimer(){
		timer.stop();
		int[] dummy = new int[2];
		boolean a = false, b= false;
		for( int i=0; i < arrow.length;i++){
			if( arrow[i].isRunning() ){
				arrow[i].stopTimer();
				dummy[0] = i;
				a = true;
			}
		}
		
		if( !a ){ dummy[0] = 999; }
		
		for( int i = 0; i < animGrid.length; i++ ){
			if( animGrid[i].isRunning() ){
				animGrid[i].stopTimer();
				dummy[1] = i;
				b = true;
			}
		}
 
		if( !b ){ dummy[1] = 999; }

		return dummy;
	}
	
	public void restartTimer( int[] counter ){
		if( counter[0] == 999 ){
			timer.restart();
		}else{
			arrow[ counter[0] ].restartTimer();
		}
		if( counter[1] == 999 ){
			timer.restart();
		}else{
			animGrid[ counter[1] ].restartTimer();
		}
	}
	
	public void startTransform( boolean b){
		startTransform = b;
	}
	
	/**
	* this method sets the transformation colors 
	*/
	public void setColors( Color backRed, Color backGreen, 
		Color gridRed, Color gridGreen, Color signalRed ){
		this.backRed = backRed;
		this.backGreen = backGreen;
		Grid.setColors( gridRed, gridGreen, signalRed );
		repaint();
	}
	
	/**
	*this method sets a new Filter
	*@param f new Filter-object which is to be set
	*/
	public void setFilter( Filter f ){
		filter = f;
		filter.fold( signal, transformed );
	}
	/**
	*this method sets the speed of the transformation
	*@param speed choose SLOW, MEDIUM or FAST to determin the speed
	*/
	public void setSpeed( int speed ){
		if( speed == SLOW ){
			//System.out.println( "TransformationSlow");
			timer.setDelay( SLOW );
			for( int i = 0; i < arrow.length; i++ ){
				arrow[i].setSpeed( AnimatedArrow.SLOW );
				//System.out.println("arrow[" + i +"].setSpeed(SLOW)" );
			}
		}else if( speed == MEDIUM){
			//System.out.println( "TransformationMedium");
			timer.setDelay( MEDIUM );
			for( int i = 0; i < arrow.length; i++ ){
				arrow[i].setSpeed( AnimatedArrow.MEDIUM );
				//System.out.println("arrow[" + i +"].setSpeed(MEDIUM)" );
			}			
		}else if( speed == FAST ){
			//System.out.println( "TransformationFast");
			timer.setDelay( FAST);
			for( int i = 0; i < arrow.length; i++ ){
				arrow[i].setSpeed( AnimatedArrow.FAST);
				//System.out.println("arrow[" + i +"].setSpeed(FAST)" );
			}
		}
				
	}//end of setSpeed
	
}