package wavelets;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;

public class AdjustmentPanel extends JPanel 
	implements ActionListener, ChangeListener{
	private PlotPanel plotPanel;	
	private JSlider slideA, slideB;
	private JTextField textA, textB;	
	private JButton resetButton;
	private double paramA, paramB;
	
	
	public AdjustmentPanel( PlotPanel plot ){
		paramA = 1;
		paramB = 0;
		
		plotPanel = plot;
		setLayout(new GridBagLayout());		
		GridBagConstraints gbc = new GridBagConstraints();	
		
		setBorder( Helper.titledBorder("Adjustment", 18) );

		slideA = new JSlider( Adjustable.VERTICAL, 1, 6000, 1000 );
		slideA.setLabelTable( Helper.createHashtableA( 6000 ) );
		slideA.setPaintLabels( true );
		slideA.setMajorTickSpacing( 1000 );
		slideA.setMinorTickSpacing( 500 );
		slideA.setPaintTicks( true );
		slideA.setToolTipText("change the dilatation factor a" );

		
		textA = Helper.addTextField( "1.0", 5, "a" );
		textA.setToolTipText("change the dilatation factor a" );

		
		JPanel panelA = new JPanel(new GridBagLayout());
		panelA.add( slideA );
		panelA.add( textA );
		panelA.setBorder( Helper.titledBorder("dilatation", 12 ) );
		panelA.setToolTipText("change the dilatation factor a" );
		
		slideB = new JSlider( Adjustable.VERTICAL, -6000, 6000, 0 );
		slideB.setLabelTable( Helper.createHashtableB( 6000 ) );
		slideB.setPaintLabels( true );
		slideB.setMajorTickSpacing( 2000 );
		slideB.setMinorTickSpacing( 1000 );
		slideB.setPaintTicks( true );
		slideB.setToolTipText("change the translation paramter b" );
		
		textB = Helper.addTextField( "0.0", 5, "b" );
		textB.setToolTipText("change the translation paramter b" );


		JPanel panelB = new JPanel(new GridBagLayout());
		panelB.add( slideB );
		panelB.add( textB );
		panelB.setBorder( Helper.titledBorder("translation", 12) );
		panelB.setToolTipText("change the translation paramter b" );
		
		resetButton = new JButton( "Reset");
		resetButton.setToolTipText("reset a and b" );
		
		gbc.anchor = GridBagConstraints.EAST;
		Helper.addToGBL( panelA, slideA, gbc, 0, 0, 1, 6, 100, 100, 0 );
		Helper.addToGBL( panelB, slideB, gbc, 0, 0, 1, 6, 100, 100, 0 );
		
		gbc.anchor = GridBagConstraints.WEST;
		Helper.addToGBL( panelA, textA, gbc, 0, 8, 1, 1, 100, 100, 3 );
		Helper.addToGBL( panelB, textB, gbc, 0, 8, 1, 1, 100, 100, 3 );

		
		Helper.addToGBL( this, panelA, gbc, 1, 1, 1, 7, 100, 100, 0 );
		Helper.addToGBL( this, panelB, gbc, 3, 1, 1, 7, 100, 100, 0 );
		
		
		//Helper.addToGBL( this, textA, gbc, 1, 8, 1, 1, 0, 0, 3);
	//	Helper.addToGBL( this, textB, gbc, 3, 8, 1, 1, 0, 0, 3);
				
		gbc.anchor = GridBagConstraints.CENTER;
		Helper.addToGBL(this, resetButton, gbc, 1, 10, 3, 1, 100, 0, 2);
		
	
		//empty horizontal Label 
		Helper.addToGBL( this, new JLabel(""), gbc, 0, 0, 5, 1, 100, 100, 0 );
		Helper.addToGBL( this, new JLabel(""), gbc, 1, 7, 3, 1, 100, 100, 0 );
		Helper.addToGBL( this, new JLabel(""), gbc, 1, 9, 3, 1, 100, 100, 0 ); 
		Helper.addToGBL( this, new JLabel(""), gbc, 0, 11, 5, 1, 100, 100, 0 );
		
		//empty  vertical Labels 
		Helper.addToGBL( this, new JLabel(""), gbc, 0, 1, 1, 10, 100, 100, 0 );
		Helper.addToGBL( this, new JLabel(""), gbc, 2, 1, 1, 6, 100,100,0);
		Helper.addToGBL( this, new JLabel(""), gbc, 4, 1, 1, 10, 100, 100, 0);
		Helper.addToGBL( this, new JLabel(""), gbc, 2, 8, 1, 1, 100, 100, 0 );
	
		
		resetButton.addActionListener( this );
		textA.addActionListener( this );
		textB.addActionListener( this );
		slideA.addChangeListener( this );
		slideB.addChangeListener( this );
	}
	
	public void stateChanged( ChangeEvent e ){
		if( e.getSource() == slideA){
			paramA = slideA.getValue()/ 1000.0;
			textA.setText( String.valueOf( paramA ) );
			plotPanel.setParam( paramA, paramB );
		}
		else if( e.getSource() == slideB){
			paramB = slideB.getValue()/1000.0;
			textB.setText( String.valueOf( paramB ) );
			plotPanel.setParam( paramA, paramB );
		}
	}

	public void reset(){
			paramA = 1;
			slideA.setValue( 1000 );
			textA.setText( "1.0" );
			paramB = 0;
			slideB.setValue( 0 );
			textB.setText( "0.0" );
	}
	
	public void actionPerformed( ActionEvent e ){
		if ( e.getSource() == resetButton ){
			reset();
		}
		else if ( e.getSource() == textA ){
			try{
				paramA = Double.parseDouble( textA.getText() );
				
				if( paramA <= 0 ){
					JOptionPane.showMessageDialog(this,
					"The dilatation factor \"a\" always has to be greater than zero",
						"Input Error",JOptionPane.ERROR_MESSAGE);
				}
				else if( paramA <= 6 ){					
					slideA.setValue( (int) (paramA * 1000) );
				}
				else{
					throw new NumberFormatException();
				}
			}catch ( NumberFormatException nfe ){
				JOptionPane.showMessageDialog(this,
					"Please insert a floating point number (0...6] ",
						"Number Format Error",JOptionPane.ERROR_MESSAGE);
			}
		}
		else if ( e.getSource() == textB ){
			try{
				paramB = Double.parseDouble( textB.getText() );
				
				if( paramB >= -6 && paramB <= 6 ){
					slideB.setValue( (int) Math.round(paramB * 1000) );
				}
				else{
					throw new NumberFormatException();
				}
			}catch ( NumberFormatException nfe ){
				JOptionPane.showMessageDialog(this,
					"Please insert a floating point number [-6...6] ",
						"Number Format Error",JOptionPane.ERROR_MESSAGE);
			}
		}

	}
	
	public static void main( String[] args ){
		JFrame f = new JFrame( "Test");
		f.setSize(300,400);
		f.getContentPane().add( new AdjustmentPanel(
					new PlotPanel( new HaarWave() ) ) );
		f.setVisible( true );
	}
	
}
		
		