package wavelets;

import javax.swing.*;
import java.awt.*;

public class PlotPanel extends JPanel{
	private double width, height; 
	private final int margintop, marginbottom, marginleft, marginright;
	private int originX, originY;
	private final Font font, bfont;
	private FontMetrics fm, fmb; 
	private double[] dposX, dnegX, dposY, dnegY, xScale, yScale;
	private Wavelet wavelet;
	private double a, b;
	
	public PlotPanel( Wavelet wave ){
	
		a = 1.0;
		b = 0.0;
		wavelet = wave;
		
		//margins of the painting area
		margintop = 30;    	//to the top
		marginbottom = 30;	//to the bottom		
		marginleft = 20;		//to the left
		marginright = 30;		//to the right
		
		font = new Font("Serif",Font.PLAIN,14);
		bfont = new Font("Serif",Font.BOLD, 14);
	}
	
	
	public void paintComponent( Graphics g ){
		super.paintComponent(g);
		
		fm = g.getFontMetrics(font);
		fmb = g.getFontMetrics(bfont);
		setBackground(Color.white);
		g.setFont(font);

		//size of the painting area
		width = getSize().width;
		height = getSize().height;
		//coordinates of the origin of the coordinatesystem 
		originX = (int) (width/2+marginleft-marginright);
		originY = (int) (height/2+margintop-marginbottom);
	
			//positions of the positive scale-values 
		//(see coordinateSystem() )
		xScale = wavelet.getXScale();
		yScale = wavelet.getYScale();
		dposX = new double[ xScale.length ];	//of the X-axis
		dposY = new double[ yScale.length ];	//of the Y-axis
		//positions of the positive scale-values
		dnegX = new double[ xScale.length ];	//of the X-axis
		dnegY = new double[ yScale.length ];	//of the Y-axis
		
		coordinateSystem( g );
	
		plot( g );
	}
	
	public void coordinateSystem( Graphics g ){
		
		drawAxisNames( g, "t", "Psi(t)" );
		drawAxis( g );
		drawArrowHeads( g );
		drawScales( g ) ;
	}//end of coordinateSystem
	
	private void drawAxisNames( Graphics g,String xName,String yName){
		
		g.setFont(bfont);
		g.setColor( Color.green.darker() );
		
		//painting the name of the X-axis
		int xPos = 
			(int) (width - marginright/2 + 8 - fmb.stringWidth(xName));
		int yPos = (int) (height/2 + margintop - marginbottom 
							+ fmb.getAscent()/2 );

		g.drawString( xName, xPos, yPos);

		//painting the name of the Y-axis
		xPos = (int) (width/2 + marginleft - marginright 
						- fmb.stringWidth(yName)/2);
		yPos = (int) (margintop/2 + fmb.getAscent()/2-4);

		g.drawString(yName, xPos, yPos);
		
		//for further use
		g.setFont(font);
		g.setColor( Color.black );
	}
	
	private void drawAxis( Graphics g ){
		//painting the X and Y axis
		
		//horizontal
		int yPos = (int) (height/2 + margintop -  marginbottom);
		g.drawLine( marginleft, yPos, 
					(int) (width - marginright), yPos);
		
		//vertical
		int xPos = (int) (width/2 + marginleft - marginright);
		g.drawLine( xPos, margintop,
					xPos, (int) (height - marginbottom) );		
	}
	
	private void drawArrowHeads( Graphics g ){
		// painting the arrowhead
		// of the X-axis
		int xPos = (int) (width - marginright); 
		int yPos = (int) (height/2 + margintop - marginbottom);

		Polygon p = new Polygon();
		p.addPoint( xPos, yPos-5 );
		p.addPoint( xPos+8, yPos );
		p.addPoint( xPos, yPos+5 );
		
		g.fillPolygon(p); 

		//of the Y-axis
		xPos = (int) (width/2 + marginleft - marginright); 
		yPos = margintop;

		p = new Polygon();
		p.addPoint( xPos-5, yPos );
		p.addPoint( xPos, yPos-10 );
		p.addPoint( xPos+5, yPos );

		g.fillPolygon(p);
	}
	
	private void drawScales( Graphics g ){
				
		String s;
		//drawing the scale
		//of the X-Axis
				
		//length of the X-axis 
		double lengthx = width-marginleft-marginright;
		
		for (int j = 0; j < xScale.length; j++)
		{	
			// positive values
			dposX[j] = Math.round(
									originX + (lengthx-10)/(2*dposX.length) * (j+1));
			g.drawLine( (int)dposX[j], originY+5, (int)dposX[j], originY-5 );

			s = String.valueOf( xScale[j] );
			g.drawString(s,
				(int) dposX[j]-fm.stringWidth(s)/2, originY+5+ fm.getAscent() );
		
			//negative values
			dnegX[j] = Math.round(
									originX- (lengthx-10)/( 2*dnegX.length ) * (j+1) );
			g.drawLine( (int)dnegX[j], originY+5, (int)dnegX[j], originY-5);

			s = String.valueOf((-xScale[j]));
			g.drawString(s,
				(int) dnegX[j]-fm.stringWidth(s)/2, originY+5+fm.getAscent() );

		} // end of j loop
		
		//drawing the scale
		//of the Y-Axis

		//length of the Y-axis 
		double lengthy = height-marginbottom-margintop;
		
		for (int j = 0; j < yScale.length; j++)
		{	
			// positive values
			dposY[j] = Math.round(
										originY - (lengthy-10)/(2*dposY.length) * (j+1));
			g.drawLine( originX-5, (int)dposY[j], originX+5, (int)dposY[j]);

			s = String.valueOf(yScale[j]);
			g.drawString(s,
				(int) originX - 7 - fm.stringWidth(s), 
					(int) dposY[j] + fm.getAscent()/2 );
		
			//negative values
			dnegY[j] = Math.round(
										originY + (lengthy-10)/(2*dnegY.length) * (j+1));
			g.drawLine( originX-5, (int)dnegY[j], originX+5, (int)dnegY[j]);

			s = String.valueOf((-yScale[j]));
			g.drawString(s,
				(int) originX - 7 - fm.stringWidth(s), 
				(int) dnegY[j] + fm.getAscent()/2  );

		} // end of j loop

		
	}
	
	public void setParam( double a, double b ){
		this.a = a;
		this.b = b;
		repaint();
	}
	
	public void setWavelet( Wavelet w ){
		wavelet = w;
		setParam( 1.0, 0.0 );
	}
	
	public void plot( Graphics g ){
		g.setColor( Color.red );
				
		//length of the X-axis 
		double lengthx = width-marginleft-marginright;
		double dy = dposY[0] - dposY[1];
		double dx = dposX[1]- dposX[0];
		double dXScale = xScale[1] - xScale[0];
		double dYScale = yScale[1] - yScale[0];
		
		
		wavelet.setA( a );
		wavelet.setB( b );
		
		//starting value of Y
		int oldX = originX;
		int oldY = (int)Math.round(originY - wavelet.formula( 0 )*dy/dYScale); 
		int newY, newX;
		
		//positiv x-values
		for( int i = 0; i < lengthx/2; i++ ){
				newY = (int) (originY - 
					wavelet.formula( i* dXScale/dx ) * dy/dYScale );
				newX =(int) ( originX + i );
				g.drawLine( oldX, oldY, newX, newY );
				oldX = newX;
				oldY = newY;
		}
		
		//negative x-values
		oldX = originX;
		oldY = (int) Math.round(originY - wavelet.formula( 0 )*dy/dYScale); 
			
		for( int i = 0; i < lengthx/2; i++ ){
			newY = (int) (originY - 
				wavelet.formula( -i* dXScale/dx ) * dy/dYScale );
			newX =(int) ( originX - i );
			g.drawLine( oldX, oldY, newX, newY );
			oldX = newX;
			oldY = newY;
		}
		
	}	

	public static void main( String[] args ){
		JFrame f = new JFrame( "test" );
		f.setSize(300,400);
		f.getContentPane().add( new PlotPanel( new Morlet() ) );
		f.setVisible( true );
	}
		
}